@@ -242,8 +242,14 @@ CCSSetting *
 ccsFindSetting (CCSPlugin * plugin, const char *name,
 		Bool isScreen, unsigned int screenNum)
 {
+    if (!plugin)
+	return NULL;
+
     PLUGIN_PRIV (plugin);
 
+    if (!name)
+	name = "";
+
     if (!pPrivate->loaded)
 	ccsLoadPluginSettings (plugin);
 
@@ -332,7 +338,6 @@ groupAdd (CCSSetting * setting, CCSPluginPrivate * p)
 }
 
 void
-
 collateGroups (CCSPluginPrivate * p)
 {
     CCSSettingList l = p->settings;
@@ -396,6 +401,7 @@ ccsFreePlugin (CCSPlugin * p)
 
     ccsSettingListFree (pPrivate->settings, TRUE);
     ccsGroupListFree (pPrivate->groups, TRUE);
+    ccsStrExtensionListFree (pPrivate->stringExtensions, TRUE);
 
     if (pPrivate->xmlFile)
 	free (pPrivate->xmlFile);
@@ -403,6 +409,11 @@ ccsFreePlugin (CCSPlugin * p)
     if (pPrivate->xmlPath)
 	free (pPrivate->xmlPath);
 
+#ifdef USE_PROTOBUF
+    if (pPrivate->pbFilePath)
+	free (pPrivate->pbFilePath);
+#endif
+
     free (pPrivate);
     free (p);
 }
@@ -425,6 +436,9 @@ ccsFreeSetting (CCSSetting * s)
     case TypeInt:
 	ccsIntDescListFree (s->info.forInt.desc, TRUE);
 	break;
+    case TypeString:
+	ccsStrRestrictionListFree (s->info.forString.restriction, TRUE);
+	break;
     case TypeList:
 	if (s->info.forList.listType == TypeInt)
 	    ccsIntDescListFree (s->info.forList.listInfo->
@@ -506,8 +520,9 @@ ccsFreePluginConflict (CCSPluginConflict * c)
 
     free (c->value);
 
-    if (c->plugins)
-	c->plugins = ccsPluginListFree (c->plugins, FALSE);
+    ccsPluginListFree (c->plugins, FALSE);
+
+    free (c);
 }
 
 void
@@ -524,6 +539,8 @@ ccsFreeBackendInfo (CCSBackendInfo * b)
 
     if (b->longDesc)
 	free (b->longDesc);
+
+    free (b);
 }
 
 void
@@ -534,6 +551,38 @@ ccsFreeIntDesc (CCSIntDesc * i)
 
     if (i->name)
 	free (i->name);
+
+    free (i);
+}
+
+void
+ccsFreeStrRestriction (CCSStrRestriction * r)
+{
+    if (!r)
+	return;
+
+    if (r->name)
+	free (r->name);
+
+    if (r->value)
+	free (r->value);
+
+    free (r);
+}
+
+void
+ccsFreeStrExtension (CCSStrExtension *e)
+{
+    if (!e)
+	return;
+
+    if (e->basePlugin)
+	free (e->basePlugin);
+
+    ccsStringListFree (e->baseSettings, TRUE);
+    ccsStrRestrictionListFree (e->restriction, TRUE);
+
+    free (e);
 }
 
 static void *
@@ -2720,3 +2769,13 @@ Bool ccsSettingIsReadOnly (CCSSetting *setting)
     return FALSE;
 }
 
+CCSStrExtensionList ccsGetPluginStrExtensions (CCSPlugin *plugin)
+{
+    PLUGIN_PRIV (plugin);
+
+    if (!pPrivate->loaded)
+	ccsLoadPluginSettings (plugin);
+
+    return pPrivate->stringExtensions;
+}
+
